#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  auth.pm - Handle authentication requests.
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#


#
#  The package this module is implementing.
#
package Xen::Monitor::Plugins::auth;


#
#  Register the plugin.
#
Xen::Monitor::Server::register_plugin("Xen::Monitor::Plugins::auth");



#
#  Minimal constructor.
#
sub new { return bless {}; }



#
#  Return the author of this plugin.
#
sub getAuthor
{
    return( 'Steve Kemp <steve@steve.org.uk>' );
}


#
#  Does this plugin require authentication?
#
sub needsAuth
{
    return 0;
}

#
#  Can this module authenticate users?
#
sub canAuth
{
    return 1;
}

#
#  Return the commands we handle.
#
sub getCommand
{
    my %commands;
    $commands{"auth"} = 1;
    return( %commands );
}


#
# Run the authentication command we're sent.
#
sub runAuth
{
    my ( $class, $server, $command, @args ) = ( @_ );

    #
    # Get the username and password.
    #
    my ( $user, $password ) = ( @args );

    #
    #  Make sure we were given a username + password.
    #
    if ( !defined( $user ) )
    {
	print "400 MISSING USER\r\n";
	return;
    }
    if ( !defined( $password ) )
    {
	print "400 MISSING PASSWORD\r\n";
	return;
    }


    #
    # Result of the auth.
    #
    my $auth = undef;

    #
    #  Open the configuration file and read the passwords.
    #
    if ( -e "/etc/argo-server/argo-server.conf" )
    {
	open( CONFIG, "<", "/etc/argo-server/argo-server.conf" );
	foreach my $line ( <CONFIG> )
	{

	    next if ( ! defined ($line) );
	    next if ( $line =~ /^\#/ );

	    #
	    #  Find lines of the form
	    # "user:pass, host1, host2, hostN"
	    #
	    if ( $line =~ /^([^:]+):([^,]+)/ )
	    {
		my $u = $1;
		my $p = $2;

		#
		#  Trim whitespace
		#
		$u =~ s/[ \t]//g;
		$p =~ s/[ \t]//g;

		if ( ( $u eq $user ) &&
		     ( $p eq $password ) )
		{
		    $auth = 1;
		}
	    }
	}
	close( CONFIG );
    }

    if ( $auth )
    {
	print "200 OK\r\n";
	$auth = $user;
	print "300 DONE\r\n";
    }
    else
    {
	print "400 AUTH FAILED\r\n";
    }
    return( $auth );
}


#
#  Return an array-ref containing virtual instance names the user may control.
#
sub getHosts 
{
    my ( $self, $username ) = ( @_ );

    my @allowed;

    #
    #  Open the configuration file and read the passwords.
    #
    if ( -e "/etc/argo-server/argo-server.conf" )
    {
	open( CONFIG, "<", "/etc/argo-server/argo-server.conf" );
	foreach my $line ( <CONFIG> )
	{

	    next if ( ! defined ($line) );
	    next if ( $line =~ /^\#/ );

	    #
	    #  Find lines of the form
	    # "user:pass, host1, host2, hostN"
	    #
	    if ( $line =~ /([^:]+):([^,]+),(.*)/ )
	    {
		my $user  = $1;
		my $pass  = $2;
		my $hosts = $3;

		$user =~ s/[ \t]//g;

		if ( ( defined( $username ) ) && ( $user eq $username ) )
		{
		    foreach my $h ( split( /,/, $hosts ) )
		    {
			
			$h =~ s/[ \t]//g;
			push @allowed, $h ;
		    }
		}
	    }
	}
	close( CONFIG );
    }

    return( \@allowed );
}


1;
